
#include "stdafx.h"




namespace network {

URLParser::URLParser()
	: valid_(false)
{}

URLParser::URLParser(const std::wstring& url)
	: valid_(false)
	, url_(url)
{
	std::wstring::const_iterator it = url.begin();
	std::wstring::const_iterator end = url.end();

	if ((it = parseProtocol(protocol_, it, end)) == end) { return; }
	if ((it = parseHost(host_, it, end)) == end) { return; }
	
	std::wstring::const_iterator tmp = it;
	if ((it = parsePort(port_, it, end)) == end) { it = tmp; }
	it = parsePath(path_, it, end);

	std::wstring::const_reverse_iterator file_begin = std::find(path_.rbegin(), path_.rend(), '/');
	if (file_begin != path_.rend())
	{
		file_.assign(file_begin.base(), path_.end());
	}

	it = parseQuery(query_, it, url.end());

	fullpath_ = path_ + query_;
	valid_ = true;
}

WORD URLParser::port()
{
	if (port_.empty())
	{
		return 80;
	}
	return _wtoi(port_.c_str());
}

const std::wstring::const_iterator URLParser::parseProtocol(
	std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end)
{
	const std::wstring marker(L"://");
	std::wstring::const_iterator last =
		std::search(begin, end, marker.begin(), marker.end());

	if (last == end) { return end; }

	dst.clear();
	dst.reserve(std::distance(begin, last));
	std::transform(begin, last, std::back_inserter(dst), tolower);
	return last + 3;
}

const std::wstring::const_iterator URLParser::parseHost(
	std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end)
{
	const std::wstring marker(L":/");
	std::wstring::const_iterator last =
		std::find_first_of(begin, end, marker.begin(), marker.end());

	if (last == end) { return end; }

	dst.clear();
	dst.reserve(std::distance(begin, last));
	std::transform(begin, last, std::back_inserter(dst), tolower);
	return last;
}

const std::wstring::const_iterator URLParser::parsePort(
	std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end)
{
	if (begin == end || begin[0] != L':')
	{
		return end;
	}

	std::wstring::const_iterator last = std::find(begin + 1, end, L'/');

	dst.assign(begin + 1, last);
	return last;
}

const std::wstring::const_iterator URLParser::parsePath(
	std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end)
{
	if (begin == end || begin[0] != L'/')
	{
		return end;
	}

	std::wstring::const_iterator last = std::find(begin + 1, end, L'?');

	dst.assign(begin, last);
	return last;
}

const std::wstring::const_iterator URLParser::parseQuery(
	std::wstring& dst, std::wstring::const_iterator begin, std::wstring::const_iterator end)
{
	if (begin == end || begin[0] != L'?')
	{
		return end;
	}

	dst.assign(begin, end);
	return end;
}

/*
HttpUrl::HttpUrl()
	: valid_(false)
{
	memset(&url_, 0, sizeof(URL_COMPONENTS));
	url_.dwStructSize = sizeof(URL_COMPONENTS);
}

HttpUrl::HttpUrl(const std::wstring& url)
	: valid_(false)
{
	WCHAR scheme[256] = {'\0'};
	WCHAR host[256] = {'\0'};
	WCHAR user[256] = {'\0'};
	WCHAR password[256] = {'\0'};
	WCHAR path[2048] = {'\0'};
	WCHAR extra[2048] = {'\0'};

	memset(&url_, 0, sizeof(URL_COMPONENTS));
	url_.dwStructSize = sizeof(URL_COMPONENTS);

	url_.lpszScheme = scheme;
	url_.dwSchemeLength = sizeof(scheme) / sizeof(WCHAR);

	url_.lpszHostName = host;
	url_.dwHostNameLength = sizeof(host) / sizeof(WCHAR);

	url_.lpszUserName = user;
	url_.dwUserNameLength = sizeof(user) / sizeof(WCHAR);

	url_.lpszPassword = password;
	url_.dwPasswordLength = sizeof(password) / sizeof(WCHAR);

	url_.lpszUrlPath = path;
	url_.dwUrlPathLength = sizeof(path) / sizeof(WCHAR);

	url_.lpszExtraInfo = extra;
	url_.dwExtraInfoLength = sizeof(extra) / sizeof(WCHAR);


	if (!WinHttpCrackUrl(url.c_str(), url.size(), 0, &url_))
	{
		return;
	}

	scheme_ = scheme;
	host_ = host;
	user_ = user;
	password_ = password;
	path_ = path;
	extra_ = extra;

	auto it = std::find(path_.rbegin(), path_.rend(), '/');
	if (it != path_.rend())
	{
		file_.assign(it.base(), path_.end());
	}

	valid_ = true;
}

std::wstring HttpUrl::scheme()
{
	return scheme_;
}

int HttpUrl::schemeType()
{
	return url_.nScheme;
}

std::wstring HttpUrl::host()
{
	return host_;
}

WORD HttpUrl::port()
{
	return url_.nPort;
}

std::wstring HttpUrl::user()
{
	return user_;
}

std::wstring HttpUrl::password()
{
	return password_;
}

std::wstring HttpUrl::path()
{
	return path_;
}

std::wstring HttpUrl::fullpath()
{
	return path_ + extra_;
}

std::wstring HttpUrl::extra()
{
	return extra_;
}

std::wstring HttpUrl::file()
{
	return file_;
}
*/
/////////////////////////////////////////////////////////////

HttpClient::~HttpClient()
{
	close();
}

bool HttpClient::open(const std::wstring& url, const std::wstring& userAgent, const std::wstring& header, int timeout)
{
	close();
	URLParser urlx(url);

	session_ = WinHttpOpen(
		userAgent.c_str(),
		WINHTTP_ACCESS_TYPE_NO_PROXY,
		WINHTTP_NO_PROXY_NAME,
		WINHTTP_NO_PROXY_BYPASS,
		0);

	if (session_)
	{
		WinHttpSetTimeouts(session_, timeout, timeout, timeout, timeout);

		connect_ = WinHttpConnect(
			session_,
			urlx.host().c_str(),
			urlx.port(),
			0);
	}

	if (connect_)
	{
		request_ = WinHttpOpenRequest(
			connect_,
			L"GET",
			urlx.fullpath().c_str(), // path
			NULL, // the HTTP version. NULL, the function uses HTTP/1.1.
			WINHTTP_NO_REFERER,
			WINHTTP_DEFAULT_ACCEPT_TYPES,
			0);
	}

	BOOL result = FALSE;

	if (request_)
	{
		result = WinHttpSendRequest(
			request_,
			header.empty() ? WINHTTP_NO_ADDITIONAL_HEADERS : header.c_str(),
			header.empty() ? 0 : -1,
			WINHTTP_NO_REQUEST_DATA,
			0, 
			0,
			0);
	}

	if (result)
	{
		result = WinHttpReceiveResponse(request_, NULL);
	}

	if (!result)
	{
		close();
		return false;
	}

	valid_ = true;
	return true;
}

bool HttpClient::post(const std::wstring& url, const std::wstring& userAgent, const char* data, DWORD size, const std::wstring& header, int timeout)
{
	close();
	URLParser urlx(url);

	session_ = WinHttpOpen(
		userAgent.c_str(),
		WINHTTP_ACCESS_TYPE_NO_PROXY,
		WINHTTP_NO_PROXY_NAME,
		WINHTTP_NO_PROXY_BYPASS,
		0);

	if (session_)
	{
		WinHttpSetTimeouts(session_, timeout, timeout, timeout, timeout);

		connect_ = WinHttpConnect(
			session_,
			urlx.host().c_str(),
			urlx.port(),
			0);
	}

	if (connect_)
	{
		request_ = WinHttpOpenRequest(
			connect_,
			L"POST",
			urlx.fullpath().c_str(), // path
			NULL, // the HTTP version. NULL, the function uses HTTP/1.1.
			WINHTTP_NO_REFERER,
			WINHTTP_DEFAULT_ACCEPT_TYPES,
			0);
	}

	BOOL result = FALSE;

	if (request_)
	{
		result = WinHttpSendRequest(
			request_,
			header.empty() ? WINHTTP_NO_ADDITIONAL_HEADERS : header.c_str(),
			header.empty() ? 0 : -1,
			WINHTTP_NO_REQUEST_DATA,
			0, 
			size,
			0);
	}

	if (result)
	{
		DWORD written = 0;
		result = WinHttpWriteData(request_, data, size, &written);
	}

	if (result)
	{
		result = WinHttpReceiveResponse(request_, NULL);
	}

	if (!result)
	{
		close();
		return false;
	}

	valid_ = true;
	return true;
}


bool HttpClient::post(const std::wstring& url, const std::wstring& userAgent, const std::vector<char>& data, const std::wstring& header, int timeout)
{
	return post(url, userAgent, data.data(), (DWORD)data.size(), header, timeout);
}

bool HttpClient::post(const std::wstring& url, const std::wstring& userAgent, const std::string& data, const std::wstring& header, int timeout)
{
	return post(url, userAgent, data.data(), (DWORD)data.size(), header, timeout);
}

int HttpClient::recv(std::vector<char>& dst)
{
	if (!valid_) { return false; }

	std::vector<char> tmp;
	DWORD size = 0, downloaded = 0;

	if (!WinHttpQueryDataAvailable(
		request_,
		&size))
	{
		DWORD error = GetLastError();
		if (error == ERROR_WINHTTP_TIMEOUT)
		{
			return STATE_TIMEOUT;
		}
		return STATE_ERROR;
	}

	if (size == 0) { return STATE_EOF; }

	tmp.resize(size);

	downloaded = 0;
	if (!WinHttpReadData(
		request_,
		tmp.data(), 
		size,
		&downloaded))
	{
		DWORD error = GetLastError();
		if (error == ERROR_WINHTTP_TIMEOUT)
		{
			return STATE_TIMEOUT;
		}
		return STATE_ERROR;
	}
	std::swap(dst, tmp);
	return STATE_OK;
}



int HttpClient::get(std::vector<char>& dst)
{
	if (!valid_) { return false; }

	int state = STATE_OK;

	while (true)
	{
		std::vector<char> tmp;
		state = recv(tmp);
		if (!tmp.empty())
		{
			dst.insert(dst.end(), tmp.begin(), tmp.end());
		}

		if (state != STATE_OK) { break; }
	}

	return state;
}

int HttpClient::read(void* dst, unsigned int size)
{
	if (!valid_) { return false; }
	
	clock_t lastTime = clock(); // ^CAEg`FbNp
	int state = STATE_OK;
	while (true)
	{
		// vTCY𖞂Ă
		if (buffer_.size() >= size)
		{
			std::memcpy(dst, buffer_.data(), size); // vTCYRs[
			// Rs[obt@폜
			std::vector<char> flip(buffer_.begin() + size, buffer_.end());
			std::swap(buffer_, flip);
			break;
		}

		// ^CAEg
		if ((clock() - lastTime) > timeout_)
		{
			return STATE_TIMEOUT;
		}

		// ǂݍ
		std::vector<char> tmp;
		state = recv(tmp);
		// ǂݍ񂾕obt@ɒǉ
		buffer_.insert(buffer_.end(), tmp.begin(), tmp.end());

		// vTCYǂݍ߂ɒf
		if (state != STATE_OK)
		{
			///////
			if (state == STATE_EOF && !buffer_.empty() && buffer_.size() <= size)
			{
				std::memcpy(dst, buffer_.data(), buffer_.size());
			}
			///////
			return state;
		}
	}
	return STATE_OK;
}

bool HttpClient::queryStatus(DWORD& code)
{
	if (request_ == NULL) { return false; }

	DWORD size = sizeof(DWORD);
	return WinHttpQueryHeaders( 
		request_, 
		WINHTTP_QUERY_STATUS_CODE | WINHTTP_QUERY_FLAG_NUMBER,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		&code, 
		&size,
		WINHTTP_NO_HEADER_INDEX
		) ? true : false;
}

bool HttpClient::queryInfo(DWORD infoLevel, std::wstring& text)
{
	if (request_ == NULL) { return false; }

	DWORD size = 0;

	WinHttpQueryHeaders(
		request_, 
		infoLevel,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		WINHTTP_NO_OUTPUT_BUFFER, 
		&size,
		WINHTTP_NO_HEADER_INDEX
		);

	DWORD err = GetLastError();
	if (err != ERROR_INSUFFICIENT_BUFFER)
	{
		return false;
	}

	std::vector<WCHAR> buff;
	buff.resize(size / sizeof(WCHAR));

	BOOL result = WinHttpQueryHeaders( 
		request_, 
		infoLevel,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		buff.data(), 
		&size,
		WINHTTP_NO_HEADER_INDEX
		);
	if (result)
	{
		text = buff.data();
		return true;
	}
	return false;
}

bool HttpClient::queryContentEncoding(std::wstring& text)
{
	return queryInfo(WINHTTP_QUERY_CONTENT_ENCODING, text);
}

bool HttpClient::queryLastModified(std::wstring& text)
{
	return queryInfo(WINHTTP_QUERY_LAST_MODIFIED, text);
}

void HttpClient::close()
{
	closeHandle(request_);
	closeHandle(connect_);
	closeHandle(session_);
	valid_ = false;
	buffer_.clear();
}

} // namespace network


/*
HttpClient::HttpClient()
	: session_(NULL)
	, connect_(NULL)
	, request_(NULL)
{}

HttpClient::~HttpClient() { close(); }

void HttpClient::close()
{
	if (session_) { WinHttpCloseHandle(session_); session_ = NULL; }
	if (connect_) { WinHttpCloseHandle(connect_); connect_ = NULL; }
	if (request_) { WinHttpCloseHandle(request_); request_ = NULL; }
}

bool HttpClient::checkPlatform()
{
	// WinHTTP T|[gĂ邩
	return WinHttpCheckPlatform() ? true : false;
}

bool HttpClient::open(const std::wstring& userAgent)
{
	close();
	// ZbVnh擾
	session_ = WinHttpOpen(
		userAgent.c_str(), 
		WINHTTP_ACCESS_TYPE_NO_PROXY,
		WINHTTP_NO_PROXY_NAME,
		WINHTTP_NO_PROXY_BYPASS,
		0);

	return session_ ? true : false;
}

bool HttpClient::open(const std::wstring& userAgent, const std::wstring& proxy)
{
	close();

	proxy_ = proxy;
	proxyBypass_ = L"<local>";
	// oCpXXg
	// vNVANZXT[o[Xg
	// <local>Ń[JʐMׂ
	// ; ŋ؂ĕݒ

	// ZbVnh擾
	session_ = WinHttpOpen(
		userAgent.c_str(), 
		WINHTTP_ACCESS_TYPE_NAMED_PROXY,
		proxy.c_str(),
		proxyBypass_.c_str(),
		0);

	return session_ ? true : false;
}


bool HttpClient::version(DWORD major, DWORD minor)
{
	if (session_ == NULL) { return false; }

	HTTP_VERSION_INFO info = {0};
	info.dwMajorVersion = major;
	info.dwMinorVersion = minor;

	return WinHttpSetOption(
		session_,
		WINHTTP_OPTION_HTTP_VERSION,
		&info,
		sizeof(info)
		) ? true : false;
}

// milliseconds
bool HttpClient::timeout(int resolve, int connect, int send, int receive)
{
	if (session_ == NULL) { return false; }

	return WinHttpSetTimeouts(
		session_,
		resolve,
		connect,
		send,
		receive
		) ? true : false;
}


bool HttpClient::proxy(const std::wstring& domain_port)
{
	if (session_ == NULL) { return false; }

	proxy_ = domain_port;
	proxyBypass_ = L"<local>";

	WINHTTP_PROXY_INFO pi = {0};
	pi.dwAccessType = WINHTTP_ACCESS_TYPE_NAMED_PROXY;
	pi.lpszProxy = const_cast<LPWSTR>(proxy_.c_str());
	pi.lpszProxyBypass = const_cast<LPWSTR>(proxyBypass_.c_str());

	return WinHttpSetOption(
		session_,
		WINHTTP_OPTION_PROXY,
		&pi,
		sizeof(pi)
		) ? true : false;
}

bool HttpClient::connect(const std::wstring& domain, INTERNET_PORT port)
{
	if (session_ == NULL) { return false; }

	connect_ = WinHttpConnect(
		session_,
		domain.c_str(),
		port,
		0);

	return connect_ ? true : false;
}

bool HttpClient::connect(const std::wstring& domain, const std::wstring& port)
{
	return connect(domain, _wtoi(port.c_str()));
}

bool HttpClient::connect(const std::wstring& domain_port)
{
	boost::basic_regex<wchar_t> regex(L"(.+):([0-9]+)");
	boost::match_results<const wchar_t*> result;

	if (!boost::regex_search(domain_port.c_str(), result, regex))
	{
		return connect(domain_port, INTERNET_DEFAULT_HTTP_PORT);
	}

	return connect(result.str(1), result.str(2));
}

bool HttpClient::openRequest(const std::wstring& path)
{
	if (connect_ == NULL) { return false; }

	request_ = WinHttpOpenRequest(
		connect_,
		L"GET",
		path.c_str(), 
		NULL,
		WINHTTP_NO_REFERER, 
		WINHTTP_DEFAULT_ACCEPT_TYPES,
		0);

	return request_ ? true : false;
}

bool HttpClient::addRequest(const std::wstring& header)
{
	// NGXgɒǉwb_[܂ޕϐւ̃|C^B
	// Ōewb_[́A
	// LbW^[/CtB[hiCR/LFjŏIKv܂B

	if (request_ == NULL) { return false; }

	return WinHttpAddRequestHeaders(
		request_, 
		header.c_str(),
		(ULONG)-1L, // ̒ -1Ń[I[
		WINHTTP_ADDREQ_FLAG_ADD // wb_[Vɐݒ肷
		) ? true : false;
}

bool HttpClient::sendRequest()
{
	if (request_ == NULL) { return false; }

	return WinHttpSendRequest(
		request_, 
		WINHTTP_NO_ADDITIONAL_HEADERS,
		0,
		WINHTTP_NO_REQUEST_DATA,
		0, 
		0,
		0
		) ? true : false;
}

bool HttpClient::receiveResponse(void)
{
	if (request_ == NULL) { return false; }
		
	return WinHttpReceiveResponse(
		request_,
		NULL // reserved and must be NULL
		) ? true : false;
}

bool HttpClient::queryInfo(DWORD infoLevel, std::wstring& text)
{
	if (request_ == NULL) { return false; }

	DWORD size = 0;

	WinHttpQueryHeaders(
		request_, 
		infoLevel,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		NULL, 
		&size,
		WINHTTP_NO_HEADER_INDEX
		);

	if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
	{
		return false;
	}

	std::vector<WCHAR> buff;
	buff.resize(size / sizeof(WCHAR));

	BOOL result = WinHttpQueryHeaders( 
		request_, 
		infoLevel,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		buff.data(), 
		&size,
		WINHTTP_NO_HEADER_INDEX
		);
	if (result)
	{
		text = buff.data();
		return true;
	}
	return false;
}

bool HttpClient::queryStatus(DWORD& code)
{
	if (request_ == NULL) { return false; }

	DWORD size = sizeof(DWORD);
	return WinHttpQueryHeaders( 
		request_, 
		WINHTTP_QUERY_STATUS_CODE | WINHTTP_QUERY_FLAG_NUMBER,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		&code, 
		&size,
		WINHTTP_NO_HEADER_INDEX
		) ? true : false;
}

bool HttpClient::queryStatus(std::wstring& text)
{
	return queryInfo(WINHTTP_QUERY_STATUS_TEXT, text);
}

bool HttpClient::queryDate(SYSTEMTIME& time)
{
	if (request_ == NULL) { return false; }

	DWORD size = sizeof(SYSTEMTIME);
	return WinHttpQueryHeaders( 
		request_, 
		WINHTTP_QUERY_DATE | WINHTTP_QUERY_FLAG_SYSTEMTIME,
		WINHTTP_HEADER_NAME_BY_INDEX, 
		&time, 
		&size,
		WINHTTP_NO_HEADER_INDEX
		) ? true : false;
}

bool HttpClient::queryDate(std::wstring& text)
{
	return queryInfo(WINHTTP_QUERY_DATE, text);
}

bool HttpClient::queryContentEncoding(std::wstring& text)
{
	return queryInfo(WINHTTP_QUERY_CONTENT_ENCODING, text);
}

bool HttpClient::readData(std::vector<char>& data)
{
	if (request_ == NULL) { return false; }

	data.clear();
	std::vector<char> buf;
	DWORD size = 0, downSize;

	while(WinHttpQueryDataAvailable(request_, &size) && size > 0)
	{
		buf.resize(size);

		if (!WinHttpReadData(request_, buf.data(), size, &downSize))
		{
			return false;
		}

		data.insert(data.end(), buf.begin(), buf.end());
	}
	return true;
}


// ======================================================================

bool Network::connect(const std::wstring& url)
{
	boost::match_results<const wchar_t*> result;
	boost::basic_regex<wchar_t> regex(L"http:\\/\\/([^\\/ ]+)(\\/[^ #]+)?");

	if (!boost::regex_search(url.c_str(), result, regex))
	{
		return false;
	}

	// open
	if (proxy_.empty())
	{
		if (!client_.open(userAgent_)) { return false; }
	}
	else
	{
		if (!client_.open(userAgent_, proxy_)) { return false; }
	}

	if (!client_.timeout(timeout_, timeout_, timeout_, timeout_))
	{
		return false;
	}

	// connect
	if (!client_.connect(result.str(1))) { return false; }

	// request
	if (!client_.openRequest(result.str(2))) { return false; }

	return true;
}

bool Network::request(const std::wstring& date)
{
	std::wstring header;

	if (gzip_)
	{
		header += L"Accept-Encoding: gzip";
	}

	if (!date.empty())
	{
		if (!header.empty())
		{
			header += L"\r\n";
		}
		header += L"If-Modified-Since: " + date;
	}

	if (!header.empty())
	{
		if (!client_.addRequest(header)) { return false; }
	}

	if (!client_.sendRequest()) { return false; }

	if (!client_.receiveResponse()) { return false; }

	return true;
}

DWORD Network::statusCode()
{
	DWORD code = 0;
	client_.queryStatus(code);
	return code;
}

std::wstring Network::statusText()
{
	std::wstring text;
	client_.queryStatus(text);
	return text;
}

std::wstring Network::date()
{
	std::wstring text;
	client_.queryDate(text);
	return text;
}

bool Network::get(const std::wstring& url)
{
	if (!connect(url)) { return false; }
	if (!request()) { return false; }

	if (statusCode() != 200) { return false; }

	//read
	std::vector<char> buff;
	if (!client_.readData(buff)) { return false; }

	if (gzip_)
	{
		std::wstring text;
		if (client_.queryContentEncoding(text))
		{
			boost::match_results<const wchar_t*> result;
			boost::basic_regex<wchar_t> regex(
				L".*gzip.*",
				boost::regbase::normal | boost::regbase::icase);

			if (boost::regex_search(text.c_str(), result, regex))
			{
				gziped_ = true;
				gzipSize_ = buff.size();

				std::vector<char> tmp;
				if (gzipDecompress(tmp, buff))
				{
					buff.swap(tmp);
				}
			}
		}
	}

	str_ = toUTF16(std::string(buff.begin(), buff.end()));
	return true;
}

std::wstring Network::str()
{
	return str_;
}

const wchar_t* Network::c_str()
{
	return str_.c_str();
}


DWORD Network::size()
{
	return str_.size();
}

bool Network::gziped()
{
	return gziped_;
}


DWORD Network::gzipSize()
{
	return gzipSize_;
}

bool Network::gzipDecompress(std::vector<char>& dst, const std::vector<char>& src)
{
	namespace io = boost::iostreams;
	try
	{
		dst.clear();
		io::filtering_istreambuf in;
		in.push(io::gzip_decompressor());
		in.push(boost::make_iterator_range(src));
		io::copy(in, io::back_inserter(dst));
	}
	catch (const io::gzip_error&) { return false; }
	catch(...) { return false; }
	return true;
}

std::wstring Network::toUTF16(const std::string& src)
{
	const int size = ::MultiByteToWideChar(CP_UTF8, 0, src.c_str(), -1, NULL, 0);

	if (size < 0) { return std::wstring(); }

	std::vector<wchar_t> buff(size);
	
	int result = ::MultiByteToWideChar(CP_UTF8, 0, src.c_str(), -1, buff.data(), size);

	if (result < 0) { return std::wstring(); }

	return buff.data();
}
*/

