#include "stdafx.h"

namespace dsp {

bool PeerCastBuilder::render(HWND hwnd, UINT msg, const std::wstring& path)
{
	boost::recursive_mutex::scoped_lock lock(mutex_);

	//DebugText(L"path:%s\n", path.c_str());

	text_.clear();

	network::PeerCast peca(path);
	if (!peca)
	{
		text_ = L"URLԈĂ܂";
		return false;
	}


	NetworkControl network;
	DecoderControl decoder;
	DecoderIDSet decoderSet;
	auto connect = [&](std::wstring& stream, const std::wstring str)->bool {
		text_ = L"ڑ" + str;
		if (!peca.connect(stream, 1000) || stream.empty())
		{
			text_ = L"ڑ..." + str;
			if (!peca.connect(stream, network.retry.timeout) || stream.empty())
			{
				text_ = L"ڑs" + str;
				return false;
			}
		}
		text_.clear();
		return true;
	};


	std::wstring stream;
	int retry = 0;
	while (true)
	{
		{
			boost::recursive_mutex::scoped_lock lock(mxcfg_);
			network = cfg_.network;
			decoder = cfg_.decoder;
			decoderSet = cfg_.decoderSet;
		}


		CString fmt;
		if (retry > 0)
		{
			fmt.Format(L" (%d/%d)", retry, network.retry.count);
		}
		std::wstring str = fmt;
		std::wstring fail;

		if (connect(stream, str))
		{
			::PostMessage(hwnd, msg, GRAPH_NOTIFY_CONNECT, 0);

			//DebugText(L"stream:%s\n", stream.c_str());

			// Xbhfꂽꍇ̓gCI
			text_ = L"\z" + str;

			//try { boost::this_thread::sleep(boost::posix_time::millisec(500)); }
			//catch(...) { text_.clear(); return false; }

			try { boost::this_thread::interruption_point(); }
			catch (...) { text_.clear(); return false; }

			SourcePtr source;
			VideoPtr video;
			AudioPtr audio;


			// FLV
			//DebugText(L"PeerCastBuilder::URLBufferSource\n");
			source.reset(new URLBufferSource(network.reader.userAgent, network.reader.timeout));
			video.reset(new EVRCustom(decoder.dxva2));
			audio.reset(new DSR());
			if (graph_.renderEx(hwnd, stream, source, video, audio, decoderSet, true))
			{
				text_.clear();
				return true;
			}

			// WMASFSourceɉȂURLnƒԒʐM҂ԂɂȂ邽ߔʂĂ
			if (source->type() != SOURCE_TYPE_ASF)
			{
				fail = L"\zs";
			}
			else if (source->isTimeout())
				//if (source->isTimeout())
			{
				fail = L"^CAEg";
			}
			else
			{
				//try { boost::this_thread::sleep(boost::posix_time::millisec(500)); }
				//catch(...) { text_.clear(); return false; }

				try { boost::this_thread::interruption_point(); }
				catch (...) { text_.clear(); return false; }

				// WMV
				//DebugText(L"PeerCastBuilder::WMASFSource\n");
				source.reset(new WMASFSource());
				video.reset(new EVRCustom(decoder.dxva2));
				audio.reset(new DSR());
				if (graph_.render(hwnd, stream, source, video, audio, decoderSet))
				{
					text_.clear();
					return true;
				}

				// ǂݍݎsɃ^CAEgߔp~
				// WMV/FLV̂ݑΉ
				//try { boost::this_thread::interruption_point(); }
				//catch(...) { text_.clear(); return false; }

				// UNKNOWN
				//source.reset(new URLFileSource());
				//video.reset(new EVRCustom());
				//audio.reset(new DSR());
				//if (graph_.renderEx(hwnd, path, source, video, audio))
				//{
				//	text_.clear();
				//	return true;
				//}

				fail = L"\zs";
			}
		}
		else
		{
			fail = L"ڑs";
		}

		if (retry >= network.retry.count) { break; }
		retry++;

		for (int i = 0; i < network.retry.wait; ++i)
		{
			CString fmt;
			fmt.Format(L"%s (%d/%d) %d", fail.c_str(), retry, network.retry.count, network.retry.wait - i);
			text_ = std::wstring(fmt);

			try { boost::this_thread::sleep(boost::posix_time::seconds(1)); }
			catch (...) { text_.clear(); return false; }
		}
	}

	if (network.retry.stop)
	{
		peca.stop();
	}

	::PostMessage(hwnd, msg, GRAPH_NOTIFY_RETRY_END, 0);

	text_ = L"~";
	return false;
}


bool PeerCastBuilder::rebuild(bool bump, HWND hwnd, UINT msg, const std::wstring& path)
{
	if (bump)
	{
		network::PeerCast peca(path);
		if (peca)
		{
			peca.bump();
			addLog(L"PeerCastBuilder::rebuild (Bump)");
		}
	}

	// Đڑҋ@
	try { boost::this_thread::sleep(boost::posix_time::seconds(1)); }
	catch (...) { return false; }

	if (!build(hwnd, msg, path)) { return false; }
	return true;
}

bool PeerCastBuilder::onAbort(HWND hwnd, UINT msg, const std::wstring& path)
{
	NetworkControl network;
	TypeControl type;
	{
		boost::recursive_mutex::scoped_lock lock(mxcfg_);
		network = cfg_.network;
		type = cfg_.type;
	}

	bool bump = false;

	if (type.flv.loop)
	{
		if (bump_.update(timeGetTime()))
		{
			bump = true;
			bump_.count = 0;
		}
	}
	else
	{
		bump = network.rebuild.enable && bump_.count >= network.rebuild.count;
		if (bump) { bump_.update(timeGetTime()); bump_.count = 0; }
		else { bump_.count++; }
	}

	addLog(L"PeerCastBuilder::onAbort (Bump Count:%d)", bump_.count);
	bool result = rebuild(bump, hwnd, msg, path);
	text_.clear();
	return result;
}

bool PeerCastBuilder::onComplete(HWND hwnd, UINT msg, const std::wstring& path)
{
	NetworkControl network;
	{
		boost::recursive_mutex::scoped_lock lock(mxcfg_);
		network = cfg_.network;
	}

	bool bump = network.rebuild.enable && bump_.count >= network.rebuild.count;
	if (bump) { bump_.update(timeGetTime()); bump_.count = 0; }
	else { bump_.count++; }

	addLog(L"PeerCastBuilder::onComplete (Bump Count:%d)", bump_.count);
	bool result = rebuild(bump, hwnd, msg, path);
	text_.clear();
	return result;
}

bool PeerCastBuilder::onRebuild(HWND hwnd, UINT msg, const std::wstring& path)
{
	NetworkControl network;
	{
		boost::recursive_mutex::scoped_lock lock(mxcfg_);
		network = cfg_.network;
	}

	bool bump = network.rebuild.enable && bump_.count >= network.rebuild.count;
	if (bump) { bump_.update(timeGetTime()); bump_.count = 0; }
	else { bump_.count++; }

	addLog(L"PeerCastBuilder::onRebuild (Bump Count:%d)", bump_.count);
	bool result = rebuild(bump, hwnd, msg, path);
	text_.clear();
	return result;

	//bool bump = false;

	//if (bump_.update(timeGetTime()))
	//{
	//	bump = true;
	//	bump_.count = 0;
	//}

	//addLog(L"PeerCastBuilder::onRebuild (Bump Count:%d)", bump_.count);
	//bool result = rebuild(bump, hwnd, msg, path);
	//text_.clear();
	//return result;
}

} // namespace dsp
