#include "stdafx.h"


namespace network {

PeerCastManager::PeerCastManager()
{

}

PeerCastManager::~PeerCastManager()
{
	thread_.join();
	group_.join_all();
}

bool PeerCastManager::init(const std::wstring& path)
{
	peca_ = PeerCast(path);
	info_ = ChannelInfo();
	text_ = std::wstring();
	return peca_;
}

void PeerCastManager::bump()
{
	group_.create_thread(boost::bind(&PeerCastManager::bumpThread, this, peca_));
}

void PeerCastManager::stop()
{
	group_.create_thread(boost::bind(&PeerCastManager::stopThread, this, peca_));
}

void PeerCastManager::info()
{
	thread_.timed_join(0);
	if (thread_.joinable()) { return; }

	boost::thread thr(&PeerCastManager::infoThread, this, peca_);
	thread_.swap(thr);
}

void PeerCastManager::setBBSCallback(const boost::function<void()>& callback)
{
	callback_.bbs = callback;
}

void PeerCastManager::setChannelCallback(const boost::function<void(bool)>& callback)
{
	callback_.channel = callback;
}

void PeerCastManager::getName(std::wstring& name)
{
	name = info_.get().name;
}

void PeerCastManager::getContactURL(std::wstring& url)
{
	url = info_.get().url;
}

void PeerCastManager::getBitrate(int& bitrate)
{
	bitrate = info_.get().cnv.bitrate;
}


void PeerCastManager::getInfo(ChannelInfo& info)
{
	info = info_.get();
}

void PeerCastManager::getInfo(std::wstring& text)
{
	text = text_.get();
}


// ===================================================================
void PeerCastManager::bumpThread(const PeerCast& peca)
{
	if (peca) { peca.bump(); }
}

void PeerCastManager::stopThread(const PeerCast& peca)
{
	if (peca) { peca.stop(); }
}

void PeerCastManager::infoThread(const PeerCast& peca)
{
	callback_.channel.get()(true);

	DWORD time = 0;
	if (peca)
	{
		ChannelInfo info;
		if (peca.info(info))
		{
			info_ = info;

			std::wstring text;

			if (!info.name.empty())
			{
				text = info.name;
			}

			if (!info.type.empty())
			{
				text += L" (" + info.type + L")";
			}

			CString bitrate;
			bitrate.Format(L" %d kbps", info.cnv.bitrate);
			text += bitrate;

			CString relay;
			relay.Format(L" [%d/%d]", info.cnv.listeners, info.cnv.relays);
			text += std::wstring(relay);

			if (!info.cnv.genreDesc.empty())
			{
				text += L"\r\n  [" + info.cnv.genreDesc + L"]";
			}

			if (!info.comment.empty())
			{
				text += L"\r\n  u" + info.comment + L"v";
			}

			if (!info.cnv.playing.empty())
			{
				text += L"\r\n  " + info.cnv.playing;
			}

			text_ = text;

			callback_.bbs.get()();
		}
		else
		{
			text_ = L"`l擾s";
		}
	}

	callback_.channel.get()(false);
}

}; // namespace network